/*--------------------------------------------------------------------------+
$Id$
|                                                                          |
| Copyright 2011 ForTISS GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.kernel.service;

import java.util.List;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.emf.ecore.EObject;
import org.fortiss.tooling.kernel.extension.IStorageProvider;
import org.fortiss.tooling.kernel.extension.data.ITopLevelElement;
import org.fortiss.tooling.kernel.extension.data.ModelStorageError;
import org.fortiss.tooling.kernel.internal.PersistencyService;
import org.fortiss.tooling.kernel.service.listener.IPersistencyServiceListener;

/**
 * The persistency service provides the transparent access to the different
 * persistency options, namely an EMFStore or local XML files within an Eclipse
 * project. Independent of the concrete {@link IStorageProvider} variant the
 * persistency service offers a list of {@link ITopLevelElement}s and a
 * notification mechanism for changes to this list.
 * 
 * @see IStorageProvider
 * @see IEclipseResourceStorageService
 * @see ITopLevelElement
 * @see IPersistencyServiceListener
 * 
 * @author hoelzl
 * @author $Author$
 * @version $Rev$
 * @ConQAT.Rating YELLOW Hash: 85BF181200615C877956EBEC24FF1B21
 */
public interface IPersistencyService {

	/** Returns the singleton instance of the service. */
	public static final IPersistencyService INSTANCE = new PersistencyService();

	/**
	 * Returns the list of top level {@link ITopLevelElement}s provided by {@link IStorageProvider}
	 * s. All storage providers are requested to load
	 * and provide their models during the persistency initialization. Therefore
	 * this method does not have a progress monitor.
	 */
	List<ITopLevelElement> getTopLevelElements();

	/** Returns whether the given element is a top-level element. */
	boolean isTopLevelElement(EObject element);

	/** Adds a top-level element listener. */
	void addTopLevelElementListener(IPersistencyServiceListener listener);

	/** Removes a top-level element listener. */
	void removeTopLevelElementListener(IPersistencyServiceListener listener);

	/** Returns the top-level element for the given model element. */
	ITopLevelElement getTopLevelElementFor(EObject modelElement);

	/** Returns the top-level elements for the given model element list. */
	List<ITopLevelElement> getTopLevelElementsFor(List<EObject> modelElements);

	/** True if all the elements of the given list share the same top level element. */
	boolean shareTopLevelElement(List<EObject> modelElements);

	/** Returns whether the some storage provider has unsaved changes. */
	boolean isDirty();

	/** Saves any dirty top-level elements to disk. */
	void doSave(IProgressMonitor monitor);

	/**
	 * Refreshes the list of top-level elements. This method should be called by
	 * storage providers whenever their list of provided top-level elements
	 * changes.
	 */
	void refreshTopLevelElements(IStorageProvider provider);

	/** Notifies top-level element listeners about change of the given element. */
	void notifyTopLevelElementChanged(ITopLevelElement element);

	/**
	 * Adds the given EObject as a dummy top-level element. This method is
	 * intended for JUnit testing purposes only.
	 */
	public ITopLevelElement addDummyEObjectAsTopLevelElement(EObject dummy);

	/**
	 * Removes the given EObject from the dummy top-level elements. This method
	 * is intended for JUnit testing purposes only.
	 */
	public void removeDummyTopLevelElement(EObject dummy);

	/** Returns all model storage errors from all storage providers. */
	public List<ModelStorageError> getAllStorageErrors();
}
